/*
 * rb_calldist.d - measure Ruby elapsed times for types of operation.
 *                 Written for the Ruby DTrace provider.
 *
 * $Id: rb_calldist.d 28 2007-09-13 10:49:37Z brendan $
 *
 * This traces Ruby activity from all programs running on the system with
 * Ruby provider support.
 *
 * This script prints distribution plots of elapsed time for Ruby
 * operations. Use rb_calltime.d for summary reports.
 *
 * FIELDS:
 *		1		Filename of the Ruby program
 *		2		Type of call (method/obj-new/gc)
 *		3		Name of call
 *
 * Filename and method names are printed if available.
 *
 * COPYRIGHT: Copyright (c) 2007 Brendan Gregg.
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * 09-Sep-2007	Brendan Gregg	Created this.
 * 10-Dec-2008	Tom Erickson	(adapted for Chime)
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

ruby*:::function-entry
{
	self->depth++;
	self->function[self->depth] = ??TIMESTAMP??;
}

ruby*:::function-return
/self->function[self->depth]/
{
	this->elapsed = ??TIMESTAMP?? - self->function[self->depth];
	self->function[self->depth] = 0;
	this->file = basename(copyinstr(arg2));
	this->name = strjoin(strjoin(copyinstr(arg0), "::"), copyinstr(arg1));

	@types[this->file, "func", this->name] =
	    quantize(this->elapsed / 1000);

	self->depth--;
}

ruby*:::object-create-start
{
	self->object = ??TIMESTAMP??;
}

ruby*:::object-create-done
/self->object/
{
	this->elapsed = ??TIMESTAMP?? - self->object;
	self->object = 0;
	this->file = basename(copyinstr(arg1));
	this->file = this->file != NULL ? this->file : ".";

	@types[this->file, "obj-new", copyinstr(arg0)] =
	    quantize(this->elapsed / 1000);
}

ruby*:::gc-begin
{
	self->gc = ??TIMESTAMP??;
}

ruby*:::gc-end
/self->gc/
{
	this->elapsed = ??TIMESTAMP?? - self->gc;
	self->gc = 0;

	@types[".", "gc", "-"] = quantize(this->elapsed / 1000);
}

??TIMESTAMP timestamp
